@@ -1,4 +1,4 @@
-from collections import namedtuple, OrderedDict
+from collections import OrderedDict, namedtuple
 from typing import Type
 
 from hcube.api.exceptions import ConfigurationError
