@@ -58,6 +58,12 @@ class StringDimension(Dimension):
     default = ""
 
 
+class BooleanDimension(Dimension):
+
+    _type = bool
+    default = False
+
+
 class IntDimension(Dimension):
 
     _type = int
