@@ -1,3 +1,4 @@
+import itertools
 import logging
 import threading
 from collections import Counter, namedtuple
@@ -59,6 +60,7 @@ logger = logging.getLogger("hcube.backends.clickhouse")
 @dataclass
 class TableMetaParams:
     engine: str = "CollapsingMergeTree"
+    table_name: Optional[str] = None
     sign_col: str = "sign"
     primary_key: [str] = field(default_factory=list)
     sorting_key: [str] = field(default_factory=list)
@@ -128,6 +130,33 @@ class ClickhouseCubeBackend(CubeBackend):
         self._drop_table(cube)
         self._table_exists.pop(self.cube_to_table_name(cube), None)
 
+    def sync_storage(self, cube: Type[Cube], drop=False) -> None:
+        """
+        Assumes that the table already exists and makes sure columns are added or removed
+        to match the cube definition.
+        """
+        table_name = self.cube_to_table_name(cube)
+        with self.pool.get_client() as client:
+            db_cols = client.execute(f"DESCRIBE TABLE {self.database}.{table_name}")
+            db_col_names = {col[0] for col in db_cols}
+            cube_col_names = {
+                col for col in itertools.chain(cube._dimensions.keys(), cube._metrics.keys())
+            }
+            cols_to_add = cube_col_names - db_col_names
+            cols_to_remove = db_col_names - cube_col_names
+
+            if drop and cols_to_remove:
+                logger.debug(f"Dropping columns {cols_to_add} from table '{table_name}'")
+                for col in cols_to_remove:
+                    client.execute(f"ALTER TABLE {self.database}.{table_name} DROP COLUMN {col}")
+
+            if cols_to_add:
+                logger.debug(f"Adding columns {cols_to_add} to table '{table_name}'")
+                for col in cols_to_add:
+                    dim = cube._dimensions.get(col) or cube._metrics.get(col)
+                    spec = self._dim_spec(dim)
+                    client.execute(f"ALTER TABLE {self.database}.{table_name} ADD COLUMN {spec}")
+
     def store_records(self, cube: Type[Cube], records: Iterable[NamedTuple]):
         with self.pool.get_client() as client:
             client.execute(f"USE {self.database}")
@@ -467,6 +496,9 @@ class ClickhouseCubeBackend(CubeBackend):
 
     @classmethod
     def cube_to_table_name(cls, cube: Union[Type[Cube], Type[AggregatingMaterializedView]]):
+        meta = cls.get_table_meta(cube)
+        if meta.table_name:
+            return meta.table_name
         return cube.__name__
 
     def _init_table(self, cube: Type[Cube]):
@@ -477,11 +509,7 @@ class ClickhouseCubeBackend(CubeBackend):
         meta = self.get_table_meta(cube)
         fields = []
         for dim in list(cube._dimensions.values()) + list(cube._metrics.values()):
-            field = f"{dim.name} {self._ch_type(dim)}"
-            if ch_spec := dim.kwargs.get("clickhouse"):
-                if codec := ch_spec.get("compression_codec"):
-                    field += f" CODEC({codec}, LZ4)"
-            fields.append(field)
+            fields.append(self._dim_spec(dim))
         field_part = ", ".join(fields)
         # indexes
         idx_part = ", ".join([idx.definition() for idx in meta.indexes])
@@ -502,8 +530,12 @@ class ClickhouseCubeBackend(CubeBackend):
                     f"sorting_key, it will be collapsed in merge."
                 )
             sorting_key = ", ".join(dim for dim in meta.sorting_key)
-        else:
+        elif meta.use_sign_col():
+            # if we use sign column (we have collapsing merge tree), we must have a sorting key
+            # with all dimensions otherwise rows would collapse
             sorting_key = ", ".join(dim for dim in cube._dimensions)
+        else:
+            sorting_key = ""
         # primary key
         primary_key = sorting_key
         if meta.primary_key:
@@ -536,6 +568,13 @@ class ClickhouseCubeBackend(CubeBackend):
             self._count_query(cube)
             client.execute(command)
 
+    def _dim_spec(self, dim) -> str:
+        field = f"{dim.name} {self._ch_type(dim)}"
+        if ch_spec := dim.kwargs.get("clickhouse"):
+            if codec := ch_spec.get("compression_codec"):
+                field += f" CODEC({codec}, LZ4)"
+        return field
+
     def _ch_type(self, dimension: Union[Dimension, Metric]) -> str:
         if isinstance(dimension, ArrayDimension):
             subtype = self._ch_type(dimension.dimension)
